'use strict';

var web3_js = require('@solana/web3.js');

// src/generated/accounts/feeVault.ts
async function findFeeVaultPda(seeds, config = {}) {
  const {
    programAddress = "TFEEgwDP6nn1s8mMX2tTNPPz8j2VomkphLUmyxKm17A"
  } = config;
  return await web3_js.getProgramDerivedAddress({
    programAddress,
    seeds: [
      web3_js.getUtf8Encoder().encode("fee_vault"),
      web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 1).encode(seeds.shard)
    ]
  });
}

// src/generated/accounts/feeVault.ts
function getFeeVaultEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([192, 178, 69, 232, 58, 149, 157, 132])
    })
  );
}
function getFeeVaultDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)]
  ]);
}
function getFeeVaultCodec() {
  return web3_js.combineCodec(getFeeVaultEncoder(), getFeeVaultDecoder());
}
function decodeFeeVault(encodedAccount) {
  return web3_js.decodeAccount(
    encodedAccount,
    getFeeVaultDecoder()
  );
}
async function fetchFeeVault(rpc, address, config) {
  const maybeAccount = await fetchMaybeFeeVault(rpc, address, config);
  web3_js.assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeFeeVault(rpc, address, config) {
  const maybeAccount = await web3_js.fetchEncodedAccount(rpc, address, config);
  return decodeFeeVault(maybeAccount);
}
async function fetchAllFeeVault(rpc, addresses, config) {
  const maybeAccounts = await fetchAllMaybeFeeVault(rpc, addresses, config);
  web3_js.assertAccountsExist(maybeAccounts);
  return maybeAccounts;
}
async function fetchAllMaybeFeeVault(rpc, addresses, config) {
  const maybeAccounts = await web3_js.fetchEncodedAccounts(rpc, addresses, config);
  return maybeAccounts.map((maybeAccount) => decodeFeeVault(maybeAccount));
}
function getFeeVaultSize() {
  return 8;
}
async function fetchFeeVaultFromSeeds(rpc, seeds, config = {}) {
  const maybeAccount = await fetchMaybeFeeVaultFromSeeds(rpc, seeds, config);
  web3_js.assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeFeeVaultFromSeeds(rpc, seeds, config = {}) {
  const { programAddress, ...fetchConfig } = config;
  const [address] = await findFeeVaultPda(seeds, { programAddress });
  return await fetchMaybeFeeVault(rpc, address, fetchConfig);
}

// src/generated/errors/feesProgram.ts
var FEES_PROGRAM_ERROR__ARITHMETIC_ERROR = 6e3;
var FEES_PROGRAM_ERROR__MISMATCHED_SEEDS_AND_ACCOUNTS = 6001;
var FEES_PROGRAM_ERROR__MISMATCHED_MINT = 6002;
var FEES_PROGRAM_ERROR__MISMATCHED_OWNER = 6003;
var feesProgramErrorMessages;
if (process.env.NODE_ENV !== "production") {
  feesProgramErrorMessages = {
    [FEES_PROGRAM_ERROR__ARITHMETIC_ERROR]: `Arithmetic error`,
    [FEES_PROGRAM_ERROR__MISMATCHED_MINT]: `Mismatched mint`,
    [FEES_PROGRAM_ERROR__MISMATCHED_OWNER]: `Mismatched owner`,
    [FEES_PROGRAM_ERROR__MISMATCHED_SEEDS_AND_ACCOUNTS]: `Number of seeds and accounts do not match`
  };
}
function getFeesProgramErrorMessage(code) {
  if (process.env.NODE_ENV !== "production") {
    return feesProgramErrorMessages[code];
  }
  return "Error message not available in production bundles.";
}
var FEES_PROGRAM_PROGRAM_ADDRESS = "TFEEgwDP6nn1s8mMX2tTNPPz8j2VomkphLUmyxKm17A";
var FeesProgramAccount = /* @__PURE__ */ ((FeesProgramAccount2) => {
  FeesProgramAccount2[FeesProgramAccount2["FeeVault"] = 0] = "FeeVault";
  return FeesProgramAccount2;
})(FeesProgramAccount || {});
function identifyFeesProgramAccount(account) {
  const data = account instanceof Uint8Array ? account : account.data;
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([192, 178, 69, 232, 58, 149, 157, 132])
    ),
    0
  )) {
    return 0 /* FeeVault */;
  }
  throw new Error(
    "The provided account could not be identified as a feesProgram account."
  );
}
var FeesProgramInstruction = /* @__PURE__ */ ((FeesProgramInstruction2) => {
  FeesProgramInstruction2[FeesProgramInstruction2["Collect"] = 0] = "Collect";
  FeesProgramInstruction2[FeesProgramInstruction2["CollectTokens"] = 1] = "CollectTokens";
  return FeesProgramInstruction2;
})(FeesProgramInstruction || {});
function identifyFeesProgramInstruction(instruction) {
  const data = instruction instanceof Uint8Array ? instruction : instruction.data;
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([208, 47, 194, 155, 17, 98, 82, 236])
    ),
    0
  )) {
    return 0 /* Collect */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([161, 0, 215, 226, 185, 222, 233, 109])
    ),
    0
  )) {
    return 1 /* CollectTokens */;
  }
  throw new Error(
    "The provided instruction could not be identified as a feesProgram instruction."
  );
}
function expectAddress(value) {
  if (!value) {
    throw new Error("Expected a Address.");
  }
  if (typeof value === "object" && "address" in value) {
    return value.address;
  }
  if (Array.isArray(value)) {
    return value[0];
  }
  return value;
}
function getAccountMetaFactory(programAddress, optionalAccountStrategy) {
  return (account) => {
    if (!account.value) {
      return Object.freeze({
        address: programAddress,
        role: web3_js.AccountRole.READONLY
      });
    }
    const writableRole = account.isWritable ? web3_js.AccountRole.WRITABLE : web3_js.AccountRole.READONLY;
    return Object.freeze({
      address: expectAddress(account.value),
      role: isTransactionSigner(account.value) ? web3_js.upgradeRoleToSigner(writableRole) : writableRole,
      ...isTransactionSigner(account.value) ? { signer: account.value } : {}
    });
  };
}
function isTransactionSigner(value) {
  return !!value && typeof value === "object" && "address" in value && web3_js.isTransactionSigner(value);
}
function getFeeSeedsEncoder() {
  return web3_js.getStructEncoder([
    ["shard", web3_js.getU8Encoder()],
    ["bump", web3_js.getU8Encoder()]
  ]);
}
function getFeeSeedsDecoder() {
  return web3_js.getStructDecoder([
    ["shard", web3_js.getU8Decoder()],
    ["bump", web3_js.getU8Decoder()]
  ]);
}
function getFeeSeedsCodec() {
  return web3_js.combineCodec(getFeeSeedsEncoder(), getFeeSeedsDecoder());
}

// src/generated/instructions/collect.ts
function getCollectInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["seeds", web3_js.getArrayEncoder(getFeeSeedsEncoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([208, 47, 194, 155, 17, 98, 82, 236])
    })
  );
}
function getCollectInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["seeds", web3_js.getArrayDecoder(getFeeSeedsDecoder())]
  ]);
}
function getCollectInstructionDataCodec() {
  return web3_js.combineCodec(
    getCollectInstructionDataEncoder(),
    getCollectInstructionDataDecoder()
  );
}
function getCollectInstruction(input) {
  const programAddress = FEES_PROGRAM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treasury: { value: input.treasury ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const remainingAccounts = args.vaults.map((address) => ({
    address,
    role: web3_js.AccountRole.WRITABLE
  }));
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treasury),
      getAccountMeta(accounts.systemProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getCollectInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseCollectInstruction(instruction) {
  if (instruction.accounts.length < 2) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treasury: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getCollectInstructionDataDecoder().decode(instruction.data)
  };
}
function getCollectTokensInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["seeds", web3_js.getArrayEncoder(getFeeSeedsEncoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([161, 0, 215, 226, 185, 222, 233, 109])
    })
  );
}
function getCollectTokensInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["seeds", web3_js.getArrayDecoder(getFeeSeedsDecoder())]
  ]);
}
function getCollectTokensInstructionDataCodec() {
  return web3_js.combineCodec(
    getCollectTokensInstructionDataEncoder(),
    getCollectTokensInstructionDataDecoder()
  );
}
function getCollectTokensInstruction(input) {
  const programAddress = FEES_PROGRAM_PROGRAM_ADDRESS;
  const originalAccounts = {
    payer: { value: input.payer ?? null, isWritable: true },
    treasury: { value: input.treasury ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    treasuryTa: { value: input.treasuryTa ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const remainingAccounts = [
    ...args.vaults.map((address) => ({ address, role: web3_js.AccountRole.WRITABLE })),
    ...args.tokenAccounts.map((address) => ({
      address,
      role: web3_js.AccountRole.WRITABLE
    }))
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.treasury),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.treasuryTa),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getCollectTokensInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseCollectTokensInstruction(instruction) {
  if (instruction.accounts.length < 7) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      payer: getNextAccount(),
      treasury: getNextAccount(),
      mint: getNextAccount(),
      treasuryTa: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getCollectTokensInstructionDataDecoder().decode(instruction.data)
  };
}

exports.FEES_PROGRAM_ERROR__ARITHMETIC_ERROR = FEES_PROGRAM_ERROR__ARITHMETIC_ERROR;
exports.FEES_PROGRAM_ERROR__MISMATCHED_MINT = FEES_PROGRAM_ERROR__MISMATCHED_MINT;
exports.FEES_PROGRAM_ERROR__MISMATCHED_OWNER = FEES_PROGRAM_ERROR__MISMATCHED_OWNER;
exports.FEES_PROGRAM_ERROR__MISMATCHED_SEEDS_AND_ACCOUNTS = FEES_PROGRAM_ERROR__MISMATCHED_SEEDS_AND_ACCOUNTS;
exports.FEES_PROGRAM_PROGRAM_ADDRESS = FEES_PROGRAM_PROGRAM_ADDRESS;
exports.FeesProgramAccount = FeesProgramAccount;
exports.FeesProgramInstruction = FeesProgramInstruction;
exports.decodeFeeVault = decodeFeeVault;
exports.fetchAllFeeVault = fetchAllFeeVault;
exports.fetchAllMaybeFeeVault = fetchAllMaybeFeeVault;
exports.fetchFeeVault = fetchFeeVault;
exports.fetchFeeVaultFromSeeds = fetchFeeVaultFromSeeds;
exports.fetchMaybeFeeVault = fetchMaybeFeeVault;
exports.fetchMaybeFeeVaultFromSeeds = fetchMaybeFeeVaultFromSeeds;
exports.findFeeVaultPda = findFeeVaultPda;
exports.getCollectInstruction = getCollectInstruction;
exports.getCollectInstructionDataCodec = getCollectInstructionDataCodec;
exports.getCollectInstructionDataDecoder = getCollectInstructionDataDecoder;
exports.getCollectInstructionDataEncoder = getCollectInstructionDataEncoder;
exports.getCollectTokensInstruction = getCollectTokensInstruction;
exports.getCollectTokensInstructionDataCodec = getCollectTokensInstructionDataCodec;
exports.getCollectTokensInstructionDataDecoder = getCollectTokensInstructionDataDecoder;
exports.getCollectTokensInstructionDataEncoder = getCollectTokensInstructionDataEncoder;
exports.getFeeSeedsCodec = getFeeSeedsCodec;
exports.getFeeSeedsDecoder = getFeeSeedsDecoder;
exports.getFeeSeedsEncoder = getFeeSeedsEncoder;
exports.getFeeVaultCodec = getFeeVaultCodec;
exports.getFeeVaultDecoder = getFeeVaultDecoder;
exports.getFeeVaultEncoder = getFeeVaultEncoder;
exports.getFeeVaultSize = getFeeVaultSize;
exports.getFeesProgramErrorMessage = getFeesProgramErrorMessage;
exports.identifyFeesProgramAccount = identifyFeesProgramAccount;
exports.identifyFeesProgramInstruction = identifyFeesProgramInstruction;
exports.parseCollectInstruction = parseCollectInstruction;
exports.parseCollectTokensInstruction = parseCollectTokensInstruction;
//# sourceMappingURL=index.js.map
//# sourceMappingURL=index.js.map