import { getProgramDerivedAddress, getUtf8Encoder, fixEncoderSize, getBytesEncoder, transformEncoder, getStructEncoder, getStructDecoder, fixDecoderSize, getBytesDecoder, combineCodec, decodeAccount, assertAccountExists, fetchEncodedAccount, assertAccountsExist, fetchEncodedAccounts, containsBytes, getU8Encoder, getU8Decoder, getArrayEncoder, getArrayDecoder, AccountRole, upgradeRoleToSigner, isTransactionSigner as isTransactionSigner$1 } from '@solana/web3.js';

// src/generated/accounts/feeVault.ts
async function findFeeVaultPda(seeds, config = {}) {
  const {
    programAddress = "TFEEgwDP6nn1s8mMX2tTNPPz8j2VomkphLUmyxKm17A"
  } = config;
  return await getProgramDerivedAddress({
    programAddress,
    seeds: [
      getUtf8Encoder().encode("fee_vault"),
      fixEncoderSize(getBytesEncoder(), 1).encode(seeds.shard)
    ]
  });
}

// src/generated/accounts/feeVault.ts
function getFeeVaultEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([192, 178, 69, 232, 58, 149, 157, 132])
    })
  );
}
function getFeeVaultDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getFeeVaultCodec() {
  return combineCodec(getFeeVaultEncoder(), getFeeVaultDecoder());
}
function decodeFeeVault(encodedAccount) {
  return decodeAccount(
    encodedAccount,
    getFeeVaultDecoder()
  );
}
async function fetchFeeVault(rpc, address, config) {
  const maybeAccount = await fetchMaybeFeeVault(rpc, address, config);
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeFeeVault(rpc, address, config) {
  const maybeAccount = await fetchEncodedAccount(rpc, address, config);
  return decodeFeeVault(maybeAccount);
}
async function fetchAllFeeVault(rpc, addresses, config) {
  const maybeAccounts = await fetchAllMaybeFeeVault(rpc, addresses, config);
  assertAccountsExist(maybeAccounts);
  return maybeAccounts;
}
async function fetchAllMaybeFeeVault(rpc, addresses, config) {
  const maybeAccounts = await fetchEncodedAccounts(rpc, addresses, config);
  return maybeAccounts.map((maybeAccount) => decodeFeeVault(maybeAccount));
}
function getFeeVaultSize() {
  return 8;
}
async function fetchFeeVaultFromSeeds(rpc, seeds, config = {}) {
  const maybeAccount = await fetchMaybeFeeVaultFromSeeds(rpc, seeds, config);
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeFeeVaultFromSeeds(rpc, seeds, config = {}) {
  const { programAddress, ...fetchConfig } = config;
  const [address] = await findFeeVaultPda(seeds, { programAddress });
  return await fetchMaybeFeeVault(rpc, address, fetchConfig);
}

// src/generated/errors/feesProgram.ts
var FEES_PROGRAM_ERROR__ARITHMETIC_ERROR = 6e3;
var FEES_PROGRAM_ERROR__MISMATCHED_SEEDS_AND_ACCOUNTS = 6001;
var FEES_PROGRAM_ERROR__MISMATCHED_MINT = 6002;
var FEES_PROGRAM_ERROR__MISMATCHED_OWNER = 6003;
var feesProgramErrorMessages;
if (process.env.NODE_ENV !== "production") {
  feesProgramErrorMessages = {
    [FEES_PROGRAM_ERROR__ARITHMETIC_ERROR]: `Arithmetic error`,
    [FEES_PROGRAM_ERROR__MISMATCHED_MINT]: `Mismatched mint`,
    [FEES_PROGRAM_ERROR__MISMATCHED_OWNER]: `Mismatched owner`,
    [FEES_PROGRAM_ERROR__MISMATCHED_SEEDS_AND_ACCOUNTS]: `Number of seeds and accounts do not match`
  };
}
function getFeesProgramErrorMessage(code) {
  if (process.env.NODE_ENV !== "production") {
    return feesProgramErrorMessages[code];
  }
  return "Error message not available in production bundles.";
}
var FEES_PROGRAM_PROGRAM_ADDRESS = "TFEEgwDP6nn1s8mMX2tTNPPz8j2VomkphLUmyxKm17A";
var FeesProgramAccount = /* @__PURE__ */ ((FeesProgramAccount2) => {
  FeesProgramAccount2[FeesProgramAccount2["FeeVault"] = 0] = "FeeVault";
  return FeesProgramAccount2;
})(FeesProgramAccount || {});
function identifyFeesProgramAccount(account) {
  const data = account instanceof Uint8Array ? account : account.data;
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([192, 178, 69, 232, 58, 149, 157, 132])
    ),
    0
  )) {
    return 0 /* FeeVault */;
  }
  throw new Error(
    "The provided account could not be identified as a feesProgram account."
  );
}
var FeesProgramInstruction = /* @__PURE__ */ ((FeesProgramInstruction2) => {
  FeesProgramInstruction2[FeesProgramInstruction2["Collect"] = 0] = "Collect";
  FeesProgramInstruction2[FeesProgramInstruction2["CollectTokens"] = 1] = "CollectTokens";
  return FeesProgramInstruction2;
})(FeesProgramInstruction || {});
function identifyFeesProgramInstruction(instruction) {
  const data = instruction instanceof Uint8Array ? instruction : instruction.data;
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([208, 47, 194, 155, 17, 98, 82, 236])
    ),
    0
  )) {
    return 0 /* Collect */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([161, 0, 215, 226, 185, 222, 233, 109])
    ),
    0
  )) {
    return 1 /* CollectTokens */;
  }
  throw new Error(
    "The provided instruction could not be identified as a feesProgram instruction."
  );
}
function expectAddress(value) {
  if (!value) {
    throw new Error("Expected a Address.");
  }
  if (typeof value === "object" && "address" in value) {
    return value.address;
  }
  if (Array.isArray(value)) {
    return value[0];
  }
  return value;
}
function getAccountMetaFactory(programAddress, optionalAccountStrategy) {
  return (account) => {
    if (!account.value) {
      return Object.freeze({
        address: programAddress,
        role: AccountRole.READONLY
      });
    }
    const writableRole = account.isWritable ? AccountRole.WRITABLE : AccountRole.READONLY;
    return Object.freeze({
      address: expectAddress(account.value),
      role: isTransactionSigner(account.value) ? upgradeRoleToSigner(writableRole) : writableRole,
      ...isTransactionSigner(account.value) ? { signer: account.value } : {}
    });
  };
}
function isTransactionSigner(value) {
  return !!value && typeof value === "object" && "address" in value && isTransactionSigner$1(value);
}
function getFeeSeedsEncoder() {
  return getStructEncoder([
    ["shard", getU8Encoder()],
    ["bump", getU8Encoder()]
  ]);
}
function getFeeSeedsDecoder() {
  return getStructDecoder([
    ["shard", getU8Decoder()],
    ["bump", getU8Decoder()]
  ]);
}
function getFeeSeedsCodec() {
  return combineCodec(getFeeSeedsEncoder(), getFeeSeedsDecoder());
}

// src/generated/instructions/collect.ts
function getCollectInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["seeds", getArrayEncoder(getFeeSeedsEncoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([208, 47, 194, 155, 17, 98, 82, 236])
    })
  );
}
function getCollectInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["seeds", getArrayDecoder(getFeeSeedsDecoder())]
  ]);
}
function getCollectInstructionDataCodec() {
  return combineCodec(
    getCollectInstructionDataEncoder(),
    getCollectInstructionDataDecoder()
  );
}
function getCollectInstruction(input) {
  const programAddress = FEES_PROGRAM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treasury: { value: input.treasury ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const remainingAccounts = args.vaults.map((address) => ({
    address,
    role: AccountRole.WRITABLE
  }));
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treasury),
      getAccountMeta(accounts.systemProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getCollectInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseCollectInstruction(instruction) {
  if (instruction.accounts.length < 2) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treasury: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getCollectInstructionDataDecoder().decode(instruction.data)
  };
}
function getCollectTokensInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["seeds", getArrayEncoder(getFeeSeedsEncoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([161, 0, 215, 226, 185, 222, 233, 109])
    })
  );
}
function getCollectTokensInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["seeds", getArrayDecoder(getFeeSeedsDecoder())]
  ]);
}
function getCollectTokensInstructionDataCodec() {
  return combineCodec(
    getCollectTokensInstructionDataEncoder(),
    getCollectTokensInstructionDataDecoder()
  );
}
function getCollectTokensInstruction(input) {
  const programAddress = FEES_PROGRAM_PROGRAM_ADDRESS;
  const originalAccounts = {
    payer: { value: input.payer ?? null, isWritable: true },
    treasury: { value: input.treasury ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    treasuryTa: { value: input.treasuryTa ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const remainingAccounts = [
    ...args.vaults.map((address) => ({ address, role: AccountRole.WRITABLE })),
    ...args.tokenAccounts.map((address) => ({
      address,
      role: AccountRole.WRITABLE
    }))
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.treasury),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.treasuryTa),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getCollectTokensInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseCollectTokensInstruction(instruction) {
  if (instruction.accounts.length < 7) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      payer: getNextAccount(),
      treasury: getNextAccount(),
      mint: getNextAccount(),
      treasuryTa: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getCollectTokensInstructionDataDecoder().decode(instruction.data)
  };
}

export { FEES_PROGRAM_ERROR__ARITHMETIC_ERROR, FEES_PROGRAM_ERROR__MISMATCHED_MINT, FEES_PROGRAM_ERROR__MISMATCHED_OWNER, FEES_PROGRAM_ERROR__MISMATCHED_SEEDS_AND_ACCOUNTS, FEES_PROGRAM_PROGRAM_ADDRESS, FeesProgramAccount, FeesProgramInstruction, decodeFeeVault, fetchAllFeeVault, fetchAllMaybeFeeVault, fetchFeeVault, fetchFeeVaultFromSeeds, fetchMaybeFeeVault, fetchMaybeFeeVaultFromSeeds, findFeeVaultPda, getCollectInstruction, getCollectInstructionDataCodec, getCollectInstructionDataDecoder, getCollectInstructionDataEncoder, getCollectTokensInstruction, getCollectTokensInstructionDataCodec, getCollectTokensInstructionDataDecoder, getCollectTokensInstructionDataEncoder, getFeeSeedsCodec, getFeeSeedsDecoder, getFeeSeedsEncoder, getFeeVaultCodec, getFeeVaultDecoder, getFeeVaultEncoder, getFeeVaultSize, getFeesProgramErrorMessage, identifyFeesProgramAccount, identifyFeesProgramInstruction, parseCollectInstruction, parseCollectTokensInstruction };
//# sourceMappingURL=index.mjs.map
//# sourceMappingURL=index.mjs.map